import pandas as pd
#import nupropagator.Global.Global as g
import math as m
import numpy as np
from scipy.interpolate import interp1d
f1=[i for i in range(1,38)]
class Cross_section:
    def __init__(self,model='CT10nlo',pdg=14,n_tt=2212):
        import importlib
        path = importlib.util.find_spec('nupropagator').submodule_search_locations[0]
        import logging
        self.log = logging.getLogger('nupropagator.Cross_section')
        self.log.debug(f'Cross_section path={path}')
        self.crossnc=pd.read_csv(path+'/cross_section/'+str(model)+'_'+str(n_tt)+'_'+str(pdg)+'_nc.dat',sep='\t', engine='python', header = None,names=['E','mean'])
        self.a=0
        self.crosscc=pd.read_csv(path+'/cross_section/'+str(model)+'_'+str(n_tt)+'_'+str(pdg)+'_cc.dat',sep='\t', engine='python', header = None, names=['E','mean'])
        self.e_max=(self.crosscc['E']).max()
        self.fcc=interp1d(np.array(self.crosscc['E']),self.crosscc['mean'],kind='cubic')
        print(self.crossnc['E'])
        self.fnc=interp1d(np.array(self.crossnc['E']),self.crossnc['mean'],kind='cubic')
        self.a=self.fcc
        self.ff=interp1d(np.array(self.crosscc['E']),self.crosscc['mean']+self.crossnc['mean'],kind='cubic')


    def get_point_energy(self):
        return np.array(self.crosscc[0])

    def get_point_sectioncc(self):
        return np.array(self.crosscc[1])

    def cross_sectioncc(self,energy):
        return (self.fcc(energy))

    def ratio_nc(self,energy):
        return self.fnc(energy)/(self.fcc(energy)+self.fnc(energy))

    def get_point_sectionnc(self):
        return np.array(self.crossnc[1])

    def cross_sectionnc(self,energy):
        return (self.fnc(energy))

    def tot(self,energy):
        return (self.fnc(energy)+self.fcc(energy))*g.cm**2

#    def tot1(self,energy):
#        if  energy<0. or
#        return (self.fnc(energy)+self.fcc(energy))


class C_s:
    def __init__(self):
        self.crossp=pd.read_csv('C10nlo_2212_14_cc.dat',sep='\t', engine='python', header=None)
        self.crossn=pd.read_csv('C10nlo_2112_14_cc.dat',sep='\t', engine='python', header=None)
        e=self.crossp[0]
        self.e_max=e.max()
        self.fp=interp1d(e,self.crossp[1],kind='cubic')
        self.fn=interp1d(e,self.crossn[1],kind='cubic')
        self.ff=interp1d(e,self.crossn[1]+self.crossp[1],kind='cubic')

class C_s1:
    def __init__(self):
        self.crossp=pd.read_csv('cross/mysec_cc_nu_cteq5.dat',sep='\t', engine='python', header=None)
        self.crossn=pd.read_csv('cross/mysec_cc_nu_cteq5.dat',sep='\t', engine='python', header=None)
        e=self.crossp[0]
        self.fp=interp1d(10**e,self.crossp[1],kind='cubic')
        self.fn=interp1d(10**e,self.crossn[1],kind='cubic')
        self.ff=interp1d(10**e,self.crossn[1]+self.crossp[1],kind='cubic')



import pandas as pd
import math as m
import numpy as np
from scipy.integrate import *
from scipy.interpolate import interp2d
from scipy.interpolate import interp1d
import collections as c
import time as t
import matplotlib.pyplot as plt
import Flux as fff
import cross as cs
import mpmath as mp
s=cs.C_s()
class zf:
    def __init__(self):
        self.crosscc=pd.read_csv('zm_factor.dat',sep='\t', engine='python', header=None,names=['energy','y','d'])
#        self.crosscc=pd.read_csv('z_factor2.dat',sep='\t', engine='python', header=None,names=['energy','y','d'])
        self.y=list(c.Counter(self.crosscc['y']).keys())
        self.e=list(c.Counter(self.crosscc['energy']).keys())
        ne=len(self.e)
        self.y=sorted(self.y)
        self.e=sorted(self.e)
        ny=len(self.y)
        self.d=np.zeros([ne,ny])
        self.e_max=(np.array(self.e)).max()
        for i in range(ne):
            a=self.crosscc[self.crosscc['energy']==self.e[i]]
            for j in range(ny):
                b=a[a['y']==self.y[j]]
                self.d[i][j]=b['d']
            print(i)
        self.f=interp2d((self.y),self.e,self.d,kind='cubic')
    
    def plot_data(self):
        e=np.logspace(1,8,num=8)
        y=self.y
        regfun=np.zeros([len(e),len(y)])
        for i in range(len(e)):
            regfun[i]=self.f(y,e[i])
            fig=plt.figure(figsize=(18,12))
            plt.plot(y,regfun[i])
            plt.title(r'Neutrino regeneration function for $E_{\nu}$ = '+str('{:1.1e}'.format(e[i])),fontsize=14)
            plt.xlabel('e',fontsize=15)
            plt.ylabel(r'$\Phi_{\nu}(E_{\nu},y)$',fontsize=15)
            plt.xscale('log')
            plt.show()
            plt.close()
    
    def plot_data1(self):
        e=np.logspace(1,8,num=80)
        x=10.**(10)
        y=np.array([x,x/2,x/5,x/10,x/20,x/50,x/100,x/1000])
        regfun=np.zeros([len(y),len(e)])
        fig=plt.figure(figsize=(18,12))
#        for i in range(len(y)):
        regfun=self.f(y,e)
        for i in range(len(y)):
            plt.plot(e,regfun[:,i],label=str(y[i]))
        plt.title(r'Neutrino regeneration function for $E_{\nu}$',fontsize=14)
        plt.xlabel('e',fontsize=15)
        plt.ylabel(r'$\Phi_{\nu}(E_{\nu},y)$',fontsize=15)
        plt.legend()
        plt.grid(True)
        plt.xscale('log')
        plt.show()
        plt.close()




import numpy as np
from nudisxs.disxs import *
import time as t
import matplotlib.pyplot as plt

def finetine_plt()-> None:
    plt.rcParams['figure.figsize'] = (8, 6)
    plt.rcParams['font.size'] = 12
    plt.rcParams['lines.linewidth'] = 2
    plt.rcParams['axes.labelsize'] = 14
    plt.rcParams['savefig.dpi'] = 300

def plot_xsec_vs_enu(enu,xsec)->None:
    finetine_plt()
    fig, axs = plt.subplots(1, 1)
    plt.plot(enu,xsec)
    fig.supxlabel(r'$\bf{E_\nu}$, [GeV]', weight="bold")
    fig.supylabel(r'$\bf{\sigma}$')
    plt.xscale('log')
    plt.yscale('log')
    plt.savefig('xs_vs_enu.pdf')

def main():
    dis = disxs()
    dis.init_pdf(name = 'nCTEQ15_208_82')
    dis.init_current('cc')
    enu = np.logspace(1,20,1000)
    tot =  np.zeros_like(enu)
    for ie,e in enumerate(enu):
        tot[ie] = dis.calculate_total(e)
    plot_xsec_vs_enu(enu,tot)


def calc_xs(ne, model = 'CT10nlo', nupdg = 16):
    dis = disxs()
    dis.init_pdf(name = model)
    dis.init_neutrino(pdg=nupdg)
    e = np.logspace(1, 20, num = ne)
    x = np.zeros(ne)
    nc_n = np.zeros(ne)
    nc_n_max = np.zeros(ne)
    cc_n = np.zeros(ne)
    cc_n_max = np.zeros(ne)
    nc_p = np.zeros(ne)
    nc_p_max = np.zeros(ne)
    cc_p = np.zeros(ne)
    cc_p_max = np.zeros(ne)

    dis.init_target('proton')
    dis.init_current('cc')
    for i in range(ne):
        tt = t.time()
        cc_p[i] = dis.calculate_total(e[i])
        w = []
        for x,wgt in dis.integrator.random():
            w.append(dis.xs_cc(e[i],x[0],x[1]))
        cc_p_max[i] = (np.array(w)).max()
        print('p  CC ', i, t.time() - tt)
    fil11 = open(str(model)+'_2212_'+str(nupdg)+'_cc.dat', 'w')
    fil12 = open(str(model)+'_2212_'+str(nupdg)+'_cc_max.dat', 'w')
    for i in range(ne):
        fil11.write(str(e[i]) + '\t' + str(cc_p[i]) + '\n')
        fil12.write(str(e[i]) + '\t' + str(cc_p_max[i]) + '\n')
    fil11.close()
    fil12.close()

    dis.init_target('neutron')
    for i in range(ne):
        tt = t.time()
        cc_n[i] = dis.calculate_total(e[i])
        w = []
        for x,wgt in dis.integrator.random():
            w.append(dis.xs_cc(e[i],x[0],x[1]))
        cc_n_max[i] = (np.array(w)).max()
        print('n  CC ', i, t.time() - tt)
    fil21 = open(str(model)+'_2112_'+str(nupdg)+'_cc.dat', 'w')
    fil22 = open(str(model)+'_2112_'+str(nupdg)+'_cc_max.dat', 'w')
    for i in range(ne):
        fil21.write(str(e[i]) + '\t' + str(cc_n[i]) + '\n')
        fil22.write(str(e[i]) + '\t' + str(cc_n_max[i]) + '\n')
    fil21.close()
    fil22.close()

    dis.init_target('proton')
    dis.init_current('nc')
    for i in range(ne):
        tt = t.time()
        nc_p[i] = dis.calculate_total(e[i])
        w = []
        for x,wgt in dis.integrator.random():
            w.append(dis.xs_nc(e[i],x[0],x[1]))
        nc_p_max[i] = (np.array(w)).max()
        print('p  NC ', i, t.time() - tt)
    fil31 = open(str(model)+'_2212_'+str(nupdg)+'_nc.dat', 'w')
    fil32 = open(str(model)+'_2212_'+str(nupdg)+'_nc_max.dat', 'w')
    for i in range(ne):
        fil31.write(str(e[i]) + '\t' + str(nc_p[i]) + '\n')
        fil32.write(str(e[i]) + '\t' + str(nc_p_max[i]) + '\n')
    fil31.close()
    fil32.close()

    dis.init_target('neutron')
    for i in range(ne):
        tt = t.time()
        nc_n[i] = dis.calculate_total(e[i])
        w = []
        for x,wgt in dis.integrator.random():
            w.append(dis.xs_nc(e[i],x[0],x[1]))
        nc_n_max[i] = (np.array(w)).max()
        print('n  NC ', i, t.time() - tt)
    fil41 = open(str(model)+'_2112_'+str(nupdg)+'_nc.dat', 'w')
    fil42 = open(str(model)+'_2112_'+str(nupdg)+'_nc_max.dat', 'w')
    for i in range(ne):
        fil41.write(str(e[i]) + '\t' + str(nc_n[i]) + '\n')
        fil42.write(str(e[i]) + '\t' + str(nc_n_max[i]) + '\n')
    fil41.close()
    fil42.close()
   
calc_xs(200)



import numpy as np
from nupropagator.cross_section import cross as cs
from nupropagator.cross_section import cross_ground as cg
from nudisxs.disxs import *
from scipy.spatial.transform import Rotation as R

class NuGen:
    def __init__(self):
        self.nu_energy_GeV = 0.
        self.nu_direction  = [0.,0.,1.]
        self.target        = -1 # not possible value. must be configured
        self.current = {-1: 'nc', 1: 'cc'}
        self.cs_p = cs.Cross_section(self.pdf_model,14,2212)
        self.cs_n = cs.Cross_section(self.pdf_model,14,2112)
        self.cross_ground_p_NC = cg.cross_ground(self.pdf_model,'nc',self.nu_pdg,2212)
        self.cross_ground_n_NC = cg.cross_ground(self.pdf_model,'nc',self.nu_pdg,2112)
        self.cross_ground_p_CC = cg.cross_ground(self.pdf_model,'cc',self.nu_pdg,2212)
        self.cross_ground_n_CC = cg.cross_ground(self.pdf_model,'cc',self.nu_pdg,2112)
        self.cross_ground = {(1,2112): self.cross_ground_n_CC.f, (1,2212): self.cross_ground_p_CC.f,
                 (-1,2112): self.cross_ground_n_NC.f, (-1,2212): self.cross_ground_p_NC.f}
        self.xsec_xy = {1: self.dis.xs_cc, -1: self.dis.xs_nc}

    def configure(self,opts):
        self.configure_simulation_mode(opts)
        self.configure_neutrino(opts)
        self.configure_target(opts)
        import logging
        self.log = logging.getLogger('nupropagator.NuGen')
        self.log.info(f'neutrino energy mode={self.nu_energy_mode}')
        self.log.info(f'neutrino direction mode={self.nu_direction_mode}')
        self.log.info(f'target mode={self.target_mode}')
        self.log.info(f'neutrino pdg ={self.nu_pdg}')

    def configure_simulation_mode(self,opts):
        self.nu_energy_mode = opts.neutrinoPrimary_energy_mode
        self.nu_direction_mode = opts.neutrinoPrimary_direction_mode
        self.target_mode = opts.neutrinoPrimary_target_mode
        self.current_mode = opts.neutrinoPrimary_current_mode

    def configure_neutrino(self,opts):
        self.nu_pdg = opts.neutrinoPrimary_pdgid
        if self.nu_energy_mode != 'random':
            self.nu_energy_GeV = opts.neutrinoPrimary_energy_GeV
        if self.nu_direction_mode != 'random':
            self.nu_direction = opts.neutrinoPrimary_direction

    def configure_target(self,opts):#FIXME
        self.pdf_model = opts.neutrinoPrimary_pdf_model
        self.dis = disxs()
        self.dis.init_pdf(self.pdf_model)
        if self.target_mode != 'random':
            self.target = opts.neutrinoPrimary_target

    def get_neutrino_direction(self):#FIXME
        if self.nu_direction_mode == 'random':
            nu_phi = 2*np.pi*np.random.random()
            nu_cos = 2*np.random.random()-1
            nu_sin = (1-nu_cos**2)**0.5
        else:
            nu_phi = np.arctan2(self.nu_direction[1],self.nu_direction[0])
            nu_cos = self.nu_direction[2]
            nu_sin = np.sqrt(self.nu_direction[0]**2+self.nu_direction[1])

        return nu_cos, nu_sin, nu_phi

    def get_neutrino_energy(self):
        if self.nu_energy_mode != 'random':
            return self.nu_energy_GeV
        else:
            # select random
            return self.nu_energy_GeV # FIX')

    def get_target(self):
        # get nucleon FIXME. Should be more generic: not only for water
        if self.target_mode == 'random':
            a = np.random.random()
            if a<=4./9:
                self.target = 2112
                self.dis.init_target('neutron')
            else:
                self.target = 2212
                self.dis.init_target('proton')

    def get_current(self):# FIXME
        if self.current_mode == 'random':
            a = np.random.random()
            if self.target == 2212:
                mode = np.int_(a <= self.cs_p.ratio_nc(self.nu_energy_GeV))*(-1)
                mode = mode + np.int_(a> self.cs_p.ratio_nc(self.nu_energy_GeV))*1
            if self.target == 2112:
                mode = np.int_(a <= self.cs_n.ratio_nc(self.nu_energy_GeV))*(-1)
                mode = mode + np.int_(a> self.cs_n.ratio_nc(self.nu_energy_GeV))*1
        else:
            mode = 0
        return mode

    def get_xy_Bjorken_old(self,mode):
        x=y=-1.0 # impossible values
        self.dis.init_current(current[mode])
        
        def get_random(x,y,crossing,diff_xsec_xy):
            c =  crossing(self.nu_energy_GeV)*np.random.random()
            xsec = diff_xsec_xy(self.nu_energy_GeV,x,y)
            if(c-xsec >= 0): 
              return true 

        while(True):
            x = np.random.random()
            y = np.random.random()
            if get_random(x,y,self.cross[(self.current,self.target)],self.xsec_xy[self.current]):
               break 
        return x,y

    def get_xy_Bjorken_new(self,mode):#FIXME
        x = []
        y = []
        weight = []
        integ = inregrand_2dxy()
        integ.dis.init_neutrino(self.nu_pdg)
        integ.dis.current(self.current)
        integ.dis.init_target(self.target)
        integ.calculate(self.nu_energy_GeV)
        for Bj,wgt in integ.integrator.random():
            x.append(Bj[0])
            y.append(Bj[1])
            weight.append(wgt)
        x = np.array(x)
        y = np.array(y)
        weight = np.array(weight)
        xsection = np.zeros(len(x))
        xsection = integ.dis.xsdis_as_array(self.nu_energy_GeV,x,y)
        return np.array(x),np.array(y),np.array(weight),np.array(xsection)
    '''
    def get_xy_Bjorken_grond_algo(self,mode):
        x=y=-1.0 # impossible values
        while(True):
            x = np.random.random()
            y = np.random.random()
            if mode == -1:
                self.dis.init_current('nc')
                rn_p_NC =  self.cross_ground_p_NC.f(self.nu_energy_GeV)*np.random.random()
                rn_n_NC =  self.cross_ground_n_NC.f(self.nu_energy_GeV)*np.random.random()
                rn_NC = rn_p_NC*np.int_(self.target == 2212) + rn_n_NC*np.int_(self.target == 2112)
                xsec = self.dis.xs_nc(self.nu_energy_GeV,x,y)
                if(rn_NC-xsec >= 0): break
            if mode == 1:
                self.dis.init_current('cc')
                rn_p_CC =  self.cross_ground_p_CC.f(self.nu_energy_GeV)*np.random.random()
                rn_n_CC =  self.cross_ground_n_CC.f(self.nu_energy_GeV)*np.random.random()
                xsec = self.dis.xs_cc(self.nu_energy_GeV,x,y)
                rn_CC = rn_p_CC*np.int_(self.target == 2212) + rn_n_CC*np.int_(self.target == 2112)
                if(rn_CC-xsec >= 0):
                    break
        return x,y
    '''
    def get_particles_momenta(self,x, y, nu_cos, nu_sin, nu_phi):#FIXME
        self.dis.dis_kinematics(self.nu_energy_GeV,x,y)
        t = np.array([0,0,nu_phi])
        t1 = np.array([0,np.arccos(nu_cos),0])
        r = R.from_rotvec(t.T)
        r1 = R.from_rotvec(t1.T)
        self.dis.particles[0]['Px_GeV'],self.dis.particles[0]['Py_GeV'],self.dis.particles[0]['Pz_GeV'] = r.apply(r1.apply(np.array([self.dis.particles[0]['Px_GeV'],self.dis.particles[0]['Py_GeV'],self.dis.particles[0]['Pz_GeV']])))
        self.dis.particles[1]['Px_GeV'],self.dis.particles[1]['Py_GeV'],self.dis.particles[1]['Pz_GeV'] = np.array([self.nu_energy_GeV*np.cos(nu_phi)*nu_sin-self.dis.particles[0]['Px_GeV'],self.nu_energy_GeV*np.sin(nu_phi)*nu_sin-self.dis.particles[0]['Py_GeV'],self.nu_energy_GeV*nu_cos-self.dis.particles[0]['Pz_GeV']])
        final_lepton = [self.dis.particles[0]['pdgid'],self.dis.particles[0]['Px_GeV'],self.dis.particles[0]['Py_GeV'],self.dis.particles[0]['Pz_GeV'],self.dis.particles[0]['E_GeV']]
        final_nucleon = [self.dis.particles[1]['pdgid'],self.dis.particles[1]['Px_GeV'],self.dis.particles[1]['Py_GeV'],self.dis.particles[1]['Pz_GeV'],self.dis.particles[1]['E_GeV']]
        return final_lepton,final_nucleon

    def get_event(self):
        # get neutrino energy
        self.dis.init_neutrino(self.nu_pdg)
        nu_cos, nu_sin, nu_phi = self.get_neutrino_direction()
        nu_energy = self.get_neutrino_energy()
        self.get_target()
        mode = self.get_current()
        x,y = self.get_xy_Bjorken(mode)
        return self.get_particles_momenta(x, y, nu_cos, nu_sin, nu_phi)

    def get_event_fix_en(self):
        self.dis.init_neutrino(self.nu_pdg)
        nu_cos, nu_sin, nu_phi = self.get_neutrino_direction()
        nu_energy = self.get_neutrino_energy()
        self.get_target()
        mode = self.get_current()
        x,y,weight = self.get_xy_Bjorken_fix_en(mode)
        return self.get_particles_momenta(x, y, nu_cos, nu_sin, nu_phi)

    def next(self):
        return self.get_event()
